/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#include "nsISupports.idl"
#include "nsICache.idl"


interface nsICacheEntryDescriptor;

[scriptable, uuid(8eadf2ed-8cac-4961-8025-6da6d5827e74)]
interface nsICacheListener : nsISupports
{
    /**
     * Called when the requested access (or appropriate subset) is
     * acquired.  The status parameter equals NS_OK on success.
     * See nsICacheService.idl for accessGranted values.
     */
    void onCacheEntryAvailable(in nsICacheEntryDescriptor descriptor,
                               in nsCacheAccessMode       accessGranted,
                               in nsresult                status);

    /**
     * Called when nsCacheSession::DoomEntry() is completed. The status
     * parameter is NS_OK when the entry was doomed, or NS_ERROR_NOT_AVAILABLE
     * when there is no such entry.
     */
    void onCacheEntryDoomed(in nsresult status);
};
