/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
#ifndef nsLWBrkCIID_h__
#define nsLWBrkCIID_h__

// {2BF64764-997F-450D-AF96-3028D1A902B0}
#define NS_LBRK_CID \
{ 0x2bf64764, 0x997f, 0x450d, \
    { 0xaf, 0x96, 0x30, 0x28, 0xd1, 0xa9, 0x2, 0xb0 } }

#define NS_LBRK_CONTRACTID "@mozilla.org/intl/lbrk;1"

// {2BF64765-997F-450D-AF96-3028D1A902B0}
#define NS_WBRK_CID \
{ 0x2bf64765, 0x997f, 0x450d, \
    { 0xaf, 0x96, 0x30, 0x28, 0xd1, 0xa9, 0x2, 0xb0 } }

#define NS_WBRK_CONTRACTID "@mozilla.org/intl/wbrk;1"

#endif
